@@ -19,7 +19,7 @@
 # ============================================================================
 # Name        : FeedingIt.py
 # Author      : Yves Marcoz
-# Version     : 0.5.4
+# Version     : 0.6.0
 # Description : Simple RSS Reader
 # ============================================================================
 
@@ -37,8 +37,8 @@ import webkit
 import time
 import dbus
 import pickle
-from os.path import isfile, isdir
-from os import mkdir
+from os.path import isfile, isdir, exists
+from os import mkdir, remove, stat
 import sys   
 import urllib2
 import gobject
@@ -46,6 +46,7 @@ from portrait import FremantleRotation
 import threading
 import thread
 from feedingitdbus import ServerObject
+from updatedbus import UpdateServerObject
 from config import Config
 from cgi import escape
 
@@ -62,6 +63,7 @@ read_color = color_style.lookup_color('DefaultTextColor')
 del color_style
 
 CONFIGDIR="/home/user/.feedingit/"
+LOCK = CONFIGDIR + "update.lock"
 
 class AddWidgetWizard(hildon.WizardDialog):
     
@@ -121,44 +123,6 @@ class AddWidgetWizard(hildon.WizardDialog):
             return False
         else:
             return True
-
-#class GetImage(threading.Thread):
-#    def __init__(self, url, stream):
-#        threading.Thread.__init__(self)
-#        self.url = url
-#        self.stream = stream
-#    
-#    def run(self):
-#        f = urllib2.urlopen(self.url)
-#        data = f.read()
-#        f.close()
-#        self.stream.write(data)
-#        self.stream.close()
-#
-#class ImageDownloader():
-#    def __init__(self):
-#        self.images = []
-#        self.downloading = False
-#        
-#    def queueImage(self, url, stream):
-#        self.images.append((url, stream))
-#        if not self.downloading:
-#            self.downloading = True
-#            gobject.timeout_add(50, self.checkQueue)
-#        
-#    def checkQueue(self):
-#        for i in range(4-threading.activeCount()):
-#            if len(self.images) > 0:
-#                (url, stream) = self.images.pop() 
-#                GetImage(url, stream).start()
-#        if len(self.images)>0:
-#            gobject.timeout_add(200, self.checkQueue)
-#        else:
-#            self.downloading=False
-#            
-#    def stopAll(self):
-#        self.images = []
-        
         
 class Download(threading.Thread):
     def __init__(self, listing, key, config):
@@ -662,7 +626,7 @@ class DisplayFeed(hildon.StackableWindow):
         self.vbox.destroy()
         self.feed = self.listing.getFeed(self.key)
         self.displayFeed()
-        self.dbusHandler.ArticleCountUpdated()
+        self.updateDbusHandler.ArticleCountUpdated()
         
     def buttonReadAllClicked(self, button):
         for index in self.feed.getIds():
@@ -688,6 +652,13 @@ class FeedingIt:
         gobject.idle_add(self.createWindow)
         
     def createWindow(self):
+        if exists(LOCK):
+            date = stat(LOCK).st_mtime
+            if date - time.time() > 2*60*60: # If the lock file is over 2 hours, remove it
+                remove(LOCK)
+            self.pannableListing.set_label("Update in progress, please wait.")
+            gobject.timeout_add_seconds(3, self.createWindow)
+            return False
         self.listing = Listing(CONFIGDIR)
         
         self.downloadDialog = False
@@ -740,6 +711,7 @@ class FeedingIt:
         
     def enableDbus(self):
         self.dbusHandler = ServerObject(self)
+        self.updateDbusHandler = UpdateServerObject(self)
 
     def button_markAll(self, button):
         for key in self.listing.getListOfFeeds():
@@ -779,6 +751,7 @@ class FeedingIt:
         
     def button_update_clicked(self, button, key):
         if not type(self.downloadDialog).__name__=="DownloadBar":
+            self.updateDbusHandler.UpdateStarted()
             self.downloadDialog = DownloadBar(self.window, self.listing, self.listing.getListOfFeeds(), self.config )
             self.downloadDialog.connect("download-done", self.onDownloadsDone)
             self.mainVbox.pack_end(self.downloadDialog, expand=False, fill=False)
@@ -790,7 +763,8 @@ class FeedingIt:
         self.downloadDialog = False
         #self.displayListing()
         self.refreshList()
-        self.dbusHandler.ArticleCountUpdated()
+        self.updateDbusHandler.UpdateFinished()
+        self.updateDbusHandler.ArticleCountUpdated()
 
     def button_preferences_clicked(self, button):
         dialog = self.config.createDialog()
@@ -858,7 +832,7 @@ class FeedingIt:
     def onFeedClosed(self, object, key):
         self.listing.saveConfig()
         self.refreshList()
-        self.dbusHandler.ArticleCountUpdated()
+        self.updateDbusHandler.ArticleCountUpdated()
      
     def run(self):
         self.window.connect("destroy", gtk.main_quit)
@@ -892,6 +866,13 @@ class FeedingIt:
         self.button_update_clicked(None, None)
         return True
     
+    def stopUpdate(self):
+        # Not implemented in the app (see update_feeds.py)
+        try:
+            self.downloadDialog.listOfKeys = []
+        except:
+            pass
+    
     def getStatus(self):
         status = ""
         for key in self.listing.getListOfFeeds():
