@@ -37,6 +37,11 @@ dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
 from os import environ
 bus = dbus.bus.BusConnection(environ["DBUS_SESSION_BUS_ADDRESS"])
 
+color_style = gtk.rc_get_style_by_paths(gtk.settings_get_default() , 'GtkButton', 'osso-logical-colors', gtk.Button)
+active_color = color_style.lookup_color('ActiveTextColor')
+default_color = color_style.lookup_color('DefaultTextColor')
+del color_style
+
 CONFIGDIR="/home/user/.feedingit/"
 
 #DBusConnection *hd_home_plugin_item_get_dbus_connection ( HDHomePluginItem *item, DBusBusType type, DBusError *error);
@@ -51,13 +56,30 @@ class FeedingItHomePlugin(hildondesktop.HomePluginItem):
         self.set_settings(True)
         self.connect("show-settings", self.show_settings)
         self.feed_list = {}
+        self.total = 0
+        self.autoupdateID=False
         
         vbox = gtk.VBox(False, 0)
         
+        #self.button = gtk.Button()
+        self.button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        self.button.set_text("FeedingIt","")
+        self.button.set_sensitive(False)
+        self.label1 = self.button.child.child.get_children()[0].get_children()[0]
+        self.label2 = self.button.child.child.get_children()[0].get_children()[1]
+        self.label1.modify_fg(gtk.STATE_NORMAL, default_color)
+        self.label2.modify_fg(gtk.STATE_NORMAL, active_color)
+        icon_theme = gtk.icon_theme_get_default()
+        pixbuf = icon_theme.load_icon("feedingit", 22, gtk.ICON_SIZE_BUTTON)
+        image = gtk.Image()
+        image.set_from_pixbuf(pixbuf)
+        self.button.set_image(image)
+        self.button.set_image_position(gtk.POS_LEFT)
+
         #button = gtk.Button("Update")
-        #button.connect("clicked", self.update_list)
+        self.button.connect("clicked", self.button_clicked)
         #button.show_all()
-        #vbox.pack_start(button)        
+        vbox.pack_start(self.button, expand=False)        
         
         #for feed in ["Slashdot", "Engadget", "Cheez"]:
         #    self.treestore.append([feed, "0"])
@@ -81,12 +103,28 @@ class FeedingItHomePlugin(hildondesktop.HomePluginItem):
           traceback.print_exc(file=file)
           file.close()
 
+    def button_clicked(self, *widget):
+        self.button.set_sensitive(False)
+        self.label1.modify_fg(gtk.STATE_NORMAL, default_color)
+        self.label2.modify_fg(gtk.STATE_NORMAL, active_color)
+        self.update_label("Stopping")
+        remote_object = bus.get_object("org.marcoz.feedingit", # Connection name
+                               "/org/marcoz/feedingit/update" # Object's path
+                              )
+        iface = dbus.Interface(remote_object, 'org.marcoz.feedingit')
+        iface.StopUpdate()
+        
+    def update_label(self, title, value=None):
+        self.button.set_title(title)
+        if value != None:
+            self.button.set_value(value)
+
     def row_activated(self, treeview, treepath, column):
         (model, iter) = self.treeview.get_selection().get_selected()
         key = model.get_value(iter, 2)
         # Create an object that will proxy for a particular remote object.
         remote_object = bus.get_object("org.maemo.feedingit", # Connection name
-                               "/org/maemo/feedingit" # Object's path
+                               "/org/maemo/feedingit/" # Object's path
                               )
         iface = dbus.Interface(remote_object, 'org.maemo.feedingit')
         iface.OpenFeed(key)
@@ -99,6 +137,8 @@ class FeedingItHomePlugin(hildondesktop.HomePluginItem):
             self.load_config()
 
         list = []
+        oldtotal = self.total
+        self.total = 0
         #for key in listOfFeeds["feedingit-order"]:
         for key in self.feed_list.keys():
             try:
@@ -110,6 +150,7 @@ class FeedingItHomePlugin(hildondesktop.HomePluginItem):
                     if readItems[id]==False:
                         countUnread = countUnread + 1
                 list.append([self.feed_list[key], countUnread, key])
+                self.total += countUnread
             except:
                 pass
         list = sorted(list, key=lambda item: item[1], reverse=True)
@@ -117,13 +158,54 @@ class FeedingItHomePlugin(hildondesktop.HomePluginItem):
             treestore.append(item)
         self.treeview.set_model(treestore)
         self.treeview.get_selection().unselect_all()
+        if self.total > oldtotal:
+            self.update_label("%s Unread" %str(self.total), "%s more articles" %str(self.total-oldtotal))
+        else:
+            self.update_label("%s Unread" %str(self.total))
         return True
+
+    def create_selector(self, choices, setting):
+        #self.pickerDialog = hildon.PickerDialog(self.parent)
+        selector = hildon.TouchSelector(text=True)
+        index = 0
+        for item in choices:
+            iter = selector.append_text(str(item))
+            if str(self.autoupdate) == str(item): 
+                selector.set_active(0, index)
+            index += 1
+        selector.connect("changed", self.selection_changed, setting)
+        #self.pickerDialog.set_selector(selector)
+        return selector
+        
+    def selection_changed(self, selector, button, setting):
+        tmp = selector.get_current_text()
+        if tmp == "Disabled":
+            self.autoupdate = 0
+        else:
+            self.autoupdate = tmp
+        #current_selection = selector.get_current_text()
+        #if current_selection:
+        #    self.config[setting] = current_selection
+        #gobject.idle_add(self.updateButton, setting)
+        #self.saveConfig()
+        
+    def create_autoupdate_picker(self):
+            picker = hildon.PickerButton(gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+            selector = self.create_selector(["Disabled", 0.02, 0.5, 1, 2, 4, 12, 24], "autoupdate")
+            picker.set_selector(selector)
+            picker.set_title("Frequency of updates from the widget")
+            picker.set_text("Setup Feed Auto-updates","Update every %s hours" %str(self.autoupdate) )
+            picker.set_name('HildonButton-finger')
+            picker.set_alignment(0,0,1,1)
+            #self.buttons[setting] = picker
+            #vbox.pack_start(picker, expand=False)
+            return picker
         
     def show_settings(self, widget):
         file = open(CONFIGDIR+"feeds.pickle")
         listOfFeeds = pickle.load(file)
         file.close()
-        dialog = gtk.Dialog("Settings", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
+        dialog = gtk.Dialog("Choose feeds to display", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR, (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
 
         self.pannableArea = hildon.PannableArea()
         
@@ -147,12 +229,16 @@ class FeedingItHomePlugin(hildondesktop.HomePluginItem):
         self.pannableArea.add(self.treeview_settings)
         self.pannableArea.show_all()
         dialog.set_default_size(-1, 600)
+        
+        dialog.action_area.pack_start(self.create_autoupdate_picker())
+        
         dialog.show_all()
         response = dialog.run()
-        print response
+
         if response == gtk.RESPONSE_ACCEPT:
             self.feed_list = self.getItems()
         dialog.destroy()
+        self.save_config()
         self.update_list()
         #self.treeview_settings.get_selection().select_all()
         
@@ -165,38 +251,64 @@ class FeedingItHomePlugin(hildondesktop.HomePluginItem):
         return list
         
     def setupDbus(self):
-        
-        #from dbus.mainloop.glib import DBusGMainLoop
-        #dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
-        #import gobject
-        #loop = gobject.MainLoop()
-        #bus = dbus.Bus(dbus.Bus.TYPE_SESSION)
-        #bus = dbus.SessionBus(mainloop=loop)
-        #bus = dbus.Bus.get_system()
-        #bus = dbus.Bus.get_session(True)
-        #bus.set_exit_on_disconnect(False)
-        
-        #remote_object = bus.get_object("org.maemo.feedingit", # Connection name
-        #                       "/org/maemo/feedingit" # Object's path
-        #                      )
-        #iface = dbus.Interface(remote_object, 'org.maemo.feedingit')
-        #iface.connect_to_signal("ArticleCountUpdated", self.update_list)
+        bus.add_signal_receiver(self.update_list, dbus_interface="org.marcoz.feedingit",
+                        signal_name="ArticleCountUpdated", path="/org/marcoz/feedingit/update")
+        bus.add_signal_receiver(self.update_started, dbus_interface="org.marcoz.feedingit",
+                        signal_name="UpdateStarted", path="/org/marcoz/feedingit/update")
+        bus.add_signal_receiver(self.update_finished, dbus_interface="org.marcoz.feedingit",
+                        signal_name="UpdateFinished", path="/org/marcoz/feedingit/update")
 
-        bus.add_signal_receiver(self.update_list, dbus_interface="org.maemo.feedingit",
-                        signal_name="ArticleCountUpdated", path="/org/maemo/feedingit")
+    def update_started(self, *widget):
+        self.button.set_sensitive(True)
+        self.update_label("Updating...", "Click to stop update")
+
+    def update_finished(self, *widget):
+        self.button.set_sensitive(False)
+        self.update_label("Update done")
         
+    def start_update(self):
+        try:
+            if self.autoupdate >0:
+                remote_object = bus.get_object("org.marcoz.feedingit", # Connection name
+                              "/org/marcoz/feedingit/update" # Object's path
+                              )
+                iface = dbus.Interface(remote_object, 'org.marcoz.feedingit')
+                iface.UpdateAll()
+            return True
+        except:
+            import traceback
+            file = open("/home/user/.feedingit/feedingit_widget.log", "w")
+            traceback.print_exc(file=file)
+            file.close()
+
     def save_config(self):
+            from os.path import isdir
             if not isdir(CONFIGDIR):
+                from os import mkdir
                 mkdir(CONFIGDIR)
             file = open(CONFIGDIR+"widget", "w")
             pickle.dump(self.feed_list, file )
+            pickle.dump(self.autoupdate, file)
             file.close()
-            
+            self.setup_autoupdate()
+
+    def setup_autoupdate(self):
+        if (float(self.autoupdate) > 0):
+            if (not self.autoupdateID==False):
+                gobject.disconnect(self.autoupdateId)
+            self.autoupdateId = gobject.timeout_add_seconds(int(float(self.autoupdate)*3600), self.start_update)
+        else:
+            if (not self.autoupdateID==False):
+                gobject.disconnect(self.autoupdateId)
+                self.autoupdateID=False
+
     def load_config(self):
             try:
                 file = open(CONFIGDIR+"widget", "r")
                 self.feed_list = pickle.load( file )
+                self.autoupdate = pickle.load( file )
                 file.close()
+                self.setup_autoupdate()
             except:
                 file = open(CONFIGDIR+"feeds.pickle")
                 listOfFeeds = pickle.load(file)
@@ -206,6 +318,7 @@ class FeedingItHomePlugin(hildondesktop.HomePluginItem):
                 for key in listOfFeeds["feedingit-order"]:
                     self.feed_list[key] = listOfFeeds[key]["title"]
                 del listOfFeeds
+                self.autoupdate = 0
 
 
 hd_plugin_type = FeedingItHomePlugin
